import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { PageResponseDto } from '@app/common/dto/pagination.response.dto';
import { timeToMinutes } from '@app/common/helper/timeToMinutes';
import { TypeORMCustomModel } from '@app/common/models/typeOrmCustom.model';
import { BookableSpaceEntityRepository } from '@app/common/modules/booking/repositories';
import { SpaceEntity } from '@app/common/modules/space/entities/space.entity';
import { SpaceRepository } from '@app/common/modules/space/repositories/space.repository';
import {
  BadRequestException,
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { In } from 'typeorm';
import { CreateBookableSpaceDto } from '../dtos';
import { BookableSpaceRequestDto } from '../dtos/bookable-space-request.dto';
import { UpdateBookableSpaceDto } from '../dtos/update-bookable-space.dto';

@Injectable()
export class BookableSpaceService {
  constructor(
    private readonly bookableSpaceEntityRepository: BookableSpaceEntityRepository,
    private readonly spaceRepository: SpaceRepository,
  ) {}

  async create(dto: CreateBookableSpaceDto) {
    // Validate time slots first
    this.validateTimeSlot(dto.startTime, dto.endTime);

    // fetch spaces exist
    const spaces = await this.getSpacesOrFindMissing(dto.spaceUuids);

    // Validate no duplicate bookable configurations
    await this.validateNoDuplicateBookableConfigs(dto.spaceUuids);

    // Create and save bookable spaces
    return this.createBookableSpaces(spaces, dto);
  }

  async findAll(
    { active, page, size, configured, search }: BookableSpaceRequestDto,
    project: string,
  ): Promise<{
    data: BaseResponseDto['data'];
    pagination: PageResponseDto;
  }> {
    let qb = this.spaceRepository
      .createQueryBuilder('space')
      .leftJoinAndSelect('space.parent', 'parentSpace')
      .leftJoinAndSelect('space.community', 'community')
      .where('community.project = :project', { project });

    if (search) {
      qb = qb.andWhere(
        'space.spaceName ILIKE :search OR community.name ILIKE :search OR parentSpace.spaceName ILIKE :search',
        { search: `%${search}%` },
      );
    }
    if (configured) {
      qb = qb
        .leftJoinAndSelect('space.bookableConfig', 'bookableConfig')
        .andWhere('bookableConfig.uuid IS NOT NULL');
      if (active !== undefined) {
        qb = qb.andWhere('bookableConfig.active = :active', { active });
      }
    } else {
      qb = qb
        .leftJoinAndSelect('space.bookableConfig', 'bookableConfig')
        .andWhere('bookableConfig.uuid IS NULL');
    }

    const customModel = TypeORMCustomModel(this.spaceRepository);

    const { baseResponseDto, paginationResponseDto } =
      await customModel.findAll({ page, size, modelName: 'space' }, qb);
    return {
      data: baseResponseDto.data.map((space) => {
        return {
          ...space,
          virtualLocation: `${space.community?.name} - ${space.parent ? space.parent?.spaceName + ' - ' : ''}${space.spaceName}`,
        };
      }),
      pagination: paginationResponseDto,
    };
  }

  /**
   * todo: if updating availability, send to the ones who have access to this space
   * todo: if updating other fields, just send emails to all users who's bookings might be affected
   */
  async update(spaceUuid: string, dto: UpdateBookableSpaceDto) {
    // fetch spaces exist
    const space = (await this.getSpacesOrFindMissing([spaceUuid]))[0];

    if (!space.bookableConfig) {
      throw new NotFoundException(
        `Bookable configuration not found for space: ${spaceUuid}`,
      );
    }
    if (dto.startTime || dto.endTime) {
      // Validate time slots first
      this.validateTimeSlot(
        dto.startTime || space.bookableConfig.startTime,
        dto.endTime || space.bookableConfig.endTime,
      );
    }
    Object.assign(space.bookableConfig, dto);
    return this.bookableSpaceEntityRepository.save(space.bookableConfig);
  }

  /**
   * Fetch spaces by UUIDs and throw an error if any are missing
   */
  private async getSpacesOrFindMissing(
    spaceUuids: string[],
  ): Promise<SpaceEntity[]> {
    const spaces = await this.spaceRepository.find({
      where: { uuid: In(spaceUuids) },
      relations: ['bookableConfig'],
    });

    if (spaces.length !== spaceUuids.length) {
      const foundUuids = spaces.map((s) => s.uuid);
      const missingUuids = spaceUuids.filter(
        (uuid) => !foundUuids.includes(uuid),
      );
      throw new NotFoundException(
        `Spaces not found: ${missingUuids.join(', ')}`,
      );
    }

    return spaces;
  }

  /**
   * Validate there are no existing bookable configurations for these spaces
   */
  private async validateNoDuplicateBookableConfigs(
    spaceUuids: string[],
  ): Promise<void> {
    const existingBookables = await this.bookableSpaceEntityRepository.find({
      where: { space: { uuid: In(spaceUuids) } },
      relations: ['space'],
    });

    if (existingBookables.length > 0) {
      const existingUuids = [
        ...new Set(existingBookables.map((b) => b.space.uuid)),
      ];
      throw new ConflictException(
        `Bookable configuration already exists for spaces: ${existingUuids.join(', ')}`,
      );
    }
  }

  /**
   * Ensure the slot start time is before the end time
   */
  private validateTimeSlot(startTime: string, endTime: string): void {
    const start = timeToMinutes(startTime);
    const end = timeToMinutes(endTime);

    if (start >= end) {
      throw new BadRequestException(
        `End time must be after start time for slot: ${startTime}-${endTime}`,
      );
    }
  }

  /**
   * Create bookable space entries after all validations pass
   */
  private async createBookableSpaces(
    spaces: SpaceEntity[],
    dto: CreateBookableSpaceDto,
  ) {
    try {
      const entries = spaces.map((space) =>
        this.bookableSpaceEntityRepository.create({
          space,
          daysAvailable: dto.daysAvailable,
          startTime: dto.startTime,
          endTime: dto.endTime,
          points: dto.points,
        }),
      );

      return this.bookableSpaceEntityRepository.save(entries);
    } catch (error) {
      if (error.code === '23505') {
        throw new ConflictException(
          'Duplicate bookable space configuration detected',
        );
      }
      throw error;
    }
  }
}
