import {
  Controller,
  Post,
  Req,
  UseGuards,
  UploadedFile,
  UseInterceptors,
  UsePipes,
  ValidationPipe,
  Param,
} from '@nestjs/common';
import {
  ApiBearerAuth,
  ApiConsumes,
  ApiOperation,
  ApiTags,
  ApiBody,
} from '@nestjs/swagger';
import { FileInterceptor } from '@nestjs/platform-express';
import { diskStorage } from 'multer';
import { ControllerRoute } from '@app/common/constants/controller-route';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { CommissionDeviceCsvDto } from '../dto';
import { DeviceCommissionService } from '../services';
import { ProjectParam } from '@app/common/dto/project-param.dto';
import { Permissions } from 'src/decorators/permissions.decorator';
import { PermissionsGuard } from 'src/guards/permissions.guard';

@ApiTags('Commission Devices Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.DEVICE_COMMISSION.ROUTE,
})
export class DeviceCommissionController {
  constructor(private readonly commissionService: DeviceCommissionService) {}

  @UseGuards(PermissionsGuard)
  @Permissions('COMMISSION_DEVICE')
  @ApiBearerAuth()
  @Post()
  @ApiConsumes('multipart/form-data')
  @UseInterceptors(
    FileInterceptor('file', {
      storage: diskStorage({
        destination: './uploads',
        filename: (req, file, cb) => {
          cb(null, `${Date.now()}-${file.originalname}`);
        },
      }),
      fileFilter: (req, file, cb) => {
        if (!file.originalname.match(/\.(csv)$/)) {
          return cb(new Error('Only CSV files are allowed!'), false);
        }
        cb(null, true);
      },
    }),
  )
  @ApiBody({ type: CommissionDeviceCsvDto })
  @ApiOperation({
    summary: ControllerRoute.DEVICE_COMMISSION.ACTIONS.ADD_ALL_DEVICES_SUMMARY,
    description:
      ControllerRoute.DEVICE_COMMISSION.ACTIONS.ADD_ALL_DEVICES_DESCRIPTION,
  })
  @UsePipes(new ValidationPipe({ transform: true }))
  async addNewDevice(
    @UploadedFile() file: Express.Multer.File,
    @Param() param: ProjectParam,
    @Req() req: any,
  ): Promise<BaseResponseDto> {
    await this.commissionService.processCsv(param, file.path);
    return {
      message: 'CSV file received and processing started',
      success: true,
    };
  }
}
