import {
  Injectable,
  HttpException,
  HttpStatus,
  BadRequestException,
} from '@nestjs/common';
import { TuyaContext } from '@tuya/tuya-connector-nodejs';
import { ConfigService } from '@nestjs/config';
import {
  AddDoorLockOnlineInterface,
  addDeviceObjectInterface,
  createTickInterface,
  deleteTemporaryPasswordInterface,
  getPasswordInterface,
  getPasswordOfflineInterface,
} from '../interfaces/door.lock.interface';
import { DeviceRepository } from '@app/common/modules/device/repositories';
import { ProductType } from '@app/common/constants/product-type.enum';

import { PasswordEncryptionService } from './encryption.services';
import { AddDoorLockOfflineTempMultipleTimeDto } from '../dtos/add.offline-temp.dto';
import { convertKeysToCamelCase } from '@app/common/helper/camelCaseConverter';
import { UpdateDoorLockOfflineTempDto } from '../dtos/update.offline-temp.dto';
import { defaultDoorLockPass } from '@app/common/constants/default.door-lock-pass';
import { VisitorPasswordRepository } from '@app/common/modules/visitor-password/repositories';
import { DeviceService } from 'src/device/services';
import {
  DaysEnum,
  EnableDisableStatusEnum,
} from '@app/common/constants/days.enum';
import { PasswordType } from '@app/common/constants/password-type.enum';
import {
  CommonHourMinutes,
  CommonHours,
} from '@app/common/constants/hours-minutes.enum';

@Injectable()
export class DoorLockService {
  private tuya: TuyaContext;
  constructor(
    private readonly configService: ConfigService,
    private readonly deviceRepository: DeviceRepository,
    private readonly deviceService: DeviceService,
    private readonly passwordEncryptionService: PasswordEncryptionService,
    private readonly visitorPasswordRepository: VisitorPasswordRepository,
  ) {
    const accessKey = this.configService.get<string>('auth-config.ACCESS_KEY');
    const secretKey = this.configService.get<string>('auth-config.SECRET_KEY');
    const tuyaEuUrl = this.configService.get<string>('tuya-config.TUYA_EU_URL');
    this.tuya = new TuyaContext({
      baseUrl: tuyaEuUrl,
      accessKey,
      secretKey,
    });
  }
  async deleteDoorLockPassword(doorLockUuid: string, passwordId: string) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }

      const deletePass = await this.deleteDoorLockPasswordTuya(
        deviceDetails.deviceTuyaUuid,
        passwordId,
      );

      if (!deletePass.success) {
        throw new HttpException('PasswordId not found', HttpStatus.NOT_FOUND);
      }
      return deletePass;
    } catch (error) {
      throw new HttpException(
        error.message || 'Error deleting temporary password',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async deleteDoorLockPasswordTuya(
    doorLockUuid: string,
    passwordId: string,
  ): Promise<deleteTemporaryPasswordInterface> {
    try {
      const path = `/v1.0/devices/${doorLockUuid}/door-lock/temp-passwords/${passwordId}`;

      const response = await this.tuya.request({
        method: 'DELETE',
        path,
      });

      return response as deleteTemporaryPasswordInterface;
    } catch (error) {
      throw new HttpException(
        'Error deleting temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getOfflineMultipleTimeTemporaryPasswords(
    doorLockUuid: string,
    fromVisitor?: boolean,
    isExpired?: boolean,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const deviceTuyaDetails =
        await this.deviceService.getDeviceDetailsByDeviceIdTuya(
          deviceDetails.deviceTuyaUuid,
        );
      const passwords = await this.getTemporaryOfflinePasswordsTuya(
        deviceDetails.deviceTuyaUuid,
        PasswordType.MULTIPLE,
        isExpired,
      );
      if (!passwords.result.records.length && fromVisitor) {
        throw new BadRequestException();
      }
      if (passwords.result.records.length > 0) {
        return fromVisitor
          ? Promise.all(
              convertKeysToCamelCase(passwords.result.records).map(
                async (password) => {
                  const passwordFromDB = await this.getVisitorPasswordFromDB(
                    password.pwdId,
                  );
                  const timestampInSeconds = Math.floor(Date.now() / 1000);
                  return {
                    passwordId: `${password.pwdId}`,
                    invalidTime: `${password.gmtExpired}`,
                    effectiveTime: `${password.gmtStart}`,
                    createdTime: `${password.gmtCreate}`,
                    passwordName: password.pwdName,
                    passwordType: 'OFFLINE_MULTIPLE',
                    deviceUuid: doorLockUuid,
                    deviceName: deviceTuyaDetails.name,
                    passwordStatus: isExpired
                      ? 'EXPIRED'
                      : timestampInSeconds > password.gmtStart
                        ? 'EFFECTIVE'
                        : 'TO_BE_EFFECTIVE',
                    authorizerEmail: passwordFromDB
                      ? passwordFromDB.user.email
                      : 'OTHER',
                    authorizerDate: passwordFromDB
                      ? `${Math.floor(new Date(passwordFromDB.createdAt).getTime() / 1000)}`
                      : '',
                  };
                },
              ),
            )
          : convertKeysToCamelCase(passwords.result.records);
      }
      return passwords;
    } catch (error) {
      throw new HttpException(
        error.message ||
          'Error getting offline multiple time temporary passwords',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getOfflineOneTimeTemporaryPasswords(
    doorLockUuid: string,
    fromVisitor?: boolean,
    isExpired?: boolean,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);
      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const deviceTuyaDetails =
        await this.deviceService.getDeviceDetailsByDeviceIdTuya(
          deviceDetails.deviceTuyaUuid,
        );
      const passwords = await this.getTemporaryOfflinePasswordsTuya(
        deviceDetails.deviceTuyaUuid,
        'once',
        isExpired,
      );
      if (!passwords.result.records.length && fromVisitor) {
        throw new BadRequestException();
      }
      if (passwords.result.records.length > 0) {
        return fromVisitor
          ? Promise.all(
              convertKeysToCamelCase(passwords.result.records).map(
                async (password) => {
                  const passwordFromDB = await this.getVisitorPasswordFromDB(
                    password.pwdId,
                  );

                  const timestampInSeconds = Math.floor(Date.now() / 1000);
                  return {
                    passwordId: `${password.pwdId}`,
                    invalidTime: `${password.gmtExpired}`,
                    effectiveTime: `${password.gmtStart}`,
                    createdTime: `${password.gmtCreate}`,
                    passwordName: password.pwdName,
                    passwordType: 'OFFLINE_ONETIME',
                    deviceUuid: doorLockUuid,
                    deviceName: deviceTuyaDetails.name,
                    passwordStatus: isExpired
                      ? 'EXPIRED'
                      : timestampInSeconds > password.gmtStart
                        ? 'EFFECTIVE'
                        : 'TO_BE_EFFECTIVE',
                    authorizerEmail: passwordFromDB
                      ? passwordFromDB.user.email
                      : 'OTHER',
                    authorizerDate: passwordFromDB
                      ? `${Math.floor(new Date(passwordFromDB.createdAt).getTime() / 1000)}`
                      : '',
                  };
                },
              ),
            )
          : convertKeysToCamelCase(passwords.result.records);
      }

      return passwords;
    } catch (error) {
      throw new HttpException(
        error.message || 'Error getting offline one time temporary passwords',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getOnlineTemporaryPasswordsMultiple(
    doorLockUuid: string,
    fromVisitor?: boolean,
    isExpired?: boolean,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const deviceTuyaDetails =
        await this.deviceService.getDeviceDetailsByDeviceIdTuya(
          deviceDetails.deviceTuyaUuid,
        );
      const passwords = await this.getOnlineTemporaryPasswordsMultipleTuya(
        deviceDetails.deviceTuyaUuid,
        isExpired,
      );
      if (passwords.result?.length > 0) {
        const passwordFiltered = passwords.result
          .filter((item) => item.type === 0)
          .map((password: any) => {
            if (password.schedule_list?.length > 0) {
              password.schedule_list = password.schedule_list.map(
                (schedule) => {
                  schedule.working_day = this.getDaysFromWorkingDayValue(
                    schedule.working_day,
                  );
                  schedule.effective_time = this.minutesToTime(
                    schedule.effective_time,
                  );
                  schedule.invalid_time = this.minutesToTime(
                    schedule.invalid_time,
                  );
                  return schedule;
                },
              );
            }
            return password;
          });

        return fromVisitor
          ? Promise.all(
              convertKeysToCamelCase(passwordFiltered).map(async (password) => {
                const passwordFromDB = await this.getVisitorPasswordFromDB(
                  password.id,
                );
                const timestampInSeconds = Math.floor(Date.now() / 1000);
                return {
                  passwordId: `${password.id}`,
                  invalidTime: `${password.invalidTime}`,
                  effectiveTime: `${password.effectiveTime}`,
                  createdTime: '',
                  scheduleList: password?.scheduleList,
                  passwordName: password.name,
                  passwordType: 'ONLINE_MULTIPLE',
                  deviceUuid: doorLockUuid,
                  deviceName: deviceTuyaDetails.name,
                  passwordStatus: isExpired
                    ? 'EXPIRED'
                    : timestampInSeconds > password.effectiveTime
                      ? 'EFFECTIVE'
                      : 'TO_BE_EFFECTIVE',
                  authorizerEmail: passwordFromDB
                    ? passwordFromDB.user.email
                    : 'OTHER',
                  authorizerDate: passwordFromDB
                    ? `${Math.floor(new Date(passwordFromDB.createdAt).getTime() / 1000)}`
                    : '',
                };
              }),
            )
          : convertKeysToCamelCase(passwordFiltered);
      }
      if (fromVisitor) {
        throw new BadRequestException();
      }
      return passwords;
    } catch (error) {
      throw new HttpException(
        error.message || 'Error getting online temporary passwords',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getVisitorPasswordFromDB(passwordId: string) {
    try {
      return await this.visitorPasswordRepository.findOne({
        where: {
          passwordTuyaUuid: passwordId,
        },
        relations: ['user'],
        select: {
          user: {
            email: true,
          },
        },
      });
    } catch (error) {
      throw new HttpException(
        error.message || 'Error retrieving visitor password from database',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async getOnlineTemporaryPasswordsOneTime(
    doorLockUuid: string,
    fromVisitor?: boolean,
    isExpired?: boolean,
  ) {
    try {
      // Retrieve device details
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const deviceTuyaDetails =
        await this.deviceService.getDeviceDetailsByDeviceIdTuya(
          deviceDetails.deviceTuyaUuid,
        );
      // Get online temporary passwords
      const passwords = await this.getOnlineTemporaryPasswordsMultipleTuya(
        deviceDetails.deviceTuyaUuid,
        isExpired,
      );
      if (passwords.result?.length > 0) {
        // Filter and map the passwords
        const passwordFiltered = passwords.result
          .filter((item) => item.type === 1)
          .map((password: any) => {
            if (password.schedule_list?.length > 0) {
              password.schedule_list = password.schedule_list.map(
                (schedule) => {
                  schedule.working_day = this.getDaysFromWorkingDayValue(
                    schedule.working_day,
                  );
                  schedule.effective_time = this.minutesToTime(
                    schedule.effective_time,
                  );
                  schedule.invalid_time = this.minutesToTime(
                    schedule.invalid_time,
                  );
                  return schedule;
                },
              );
            }
            return password;
          });

        // Process passwords with optimized query handling
        return fromVisitor
          ? Promise.all(
              convertKeysToCamelCase(passwordFiltered).map(async (password) => {
                const passwordFromDB = await this.getVisitorPasswordFromDB(
                  password.id,
                );
                const timestampInSeconds = Math.floor(Date.now() / 1000);
                return {
                  passwordId: `${password.id}`,
                  invalidTime: `${password.invalidTime}`,
                  effectiveTime: `${password.effectiveTime}`,
                  createdTime: '',
                  passwordName: password.name,
                  passwordType: 'ONLINE_ONETIME',
                  deviceUuid: doorLockUuid,
                  deviceName: deviceTuyaDetails.name,
                  passwordStatus: isExpired
                    ? 'EXPIRED'
                    : timestampInSeconds > password.effectiveTime
                      ? 'EFFECTIVE'
                      : 'TO_BE_EFFECTIVE',
                  authorizerEmail: passwordFromDB
                    ? passwordFromDB.user.email
                    : 'OTHER',
                  authorizerDate: passwordFromDB
                    ? `${Math.floor(new Date(passwordFromDB.createdAt).getTime() / 1000)}`
                    : '',
                };
              }),
            )
          : convertKeysToCamelCase(passwordFiltered);
      }
      if (fromVisitor) {
        throw new BadRequestException();
      }
      return passwords;
    } catch (error) {
      throw new HttpException(
        error.message || 'Error getting online temporary passwords',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async getOnlineTemporaryPasswordsMultipleTuya(
    doorLockUuid: string,
    isExpired?: boolean,
  ): Promise<getPasswordInterface> {
    try {
      const path = `/v1.0/devices/${doorLockUuid}/door-lock/temp-passwords?valid=${!isExpired}`;

      const response = await this.tuya.request({
        method: 'GET',
        path,
      });

      return response as getPasswordInterface;
    } catch (error) {
      throw new HttpException(
        'Error getting temporary passwords from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getTemporaryOfflinePasswordsTuya(
    doorLockUuid: string,
    type: string,
    isExpired?: boolean,
  ): Promise<getPasswordOfflineInterface> {
    try {
      const path = `/v1.0/devices/${doorLockUuid}/door-lock/offline-temp-password?pwd_type_codes=${type}&target_status=${isExpired ? 'INEFFECTIVE' : 'EFFECTIVE'}&page_no=1&page_size=100`;

      const response = await this.tuya.request({
        method: 'GET',
        path,
      });

      return response as getPasswordOfflineInterface;
    } catch (error) {
      throw new HttpException(
        'Error getting offline temporary passwords from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async addOfflineMultipleTimeTemporaryPassword(
    addDoorLockOfflineTempMultipleTimeDto: AddDoorLockOfflineTempMultipleTimeDto,
    doorLockUuid: string,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const createOnceOfflinePass = await this.addOfflineTemporaryPasswordTuya(
        deviceDetails.deviceTuyaUuid,
        PasswordType.MULTIPLE,
        addDoorLockOfflineTempMultipleTimeDto,
      );
      if (!createOnceOfflinePass.success) {
        throw new HttpException(
          createOnceOfflinePass.msg,
          HttpStatus.BAD_REQUEST,
        );
      }
      return {
        result: createOnceOfflinePass.result,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Error adding offline temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async addOfflineOneTimeTemporaryPassword(doorLockUuid: string) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const createOnceOfflinePass = await this.addOfflineTemporaryPasswordTuya(
        deviceDetails.deviceTuyaUuid,
        'once',
        null,
      );
      if (!createOnceOfflinePass.success) {
        throw new HttpException(
          createOnceOfflinePass.msg,
          HttpStatus.BAD_REQUEST,
        );
      }
      return {
        result: createOnceOfflinePass.result,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Error adding offline temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async addOfflineTemporaryPasswordTuya(
    doorLockUuid: string,
    type: string,
    addDoorLockOfflineTempMultipleTimeDto: AddDoorLockOfflineTempMultipleTimeDto,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.1/devices/${doorLockUuid}/door-lock/offline-temp-password`;

      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          ...(type === PasswordType.MULTIPLE && {
            effective_time: addDoorLockOfflineTempMultipleTimeDto.effectiveTime,
            invalid_time: addDoorLockOfflineTempMultipleTimeDto.invalidTime,
          }),

          type,
        },
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        'Error adding offline temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async addOnlineTemporaryPassword(
    addDoorLockDto: AddDoorLockOnlineInterface,
    doorLockUuid: string,
  ) {
    try {
      const passwordData = await this.getTicketAndEncryptedPassword(
        doorLockUuid,
        addDoorLockDto.password,
      );
      if (
        !passwordData.ticketKey ||
        !passwordData.encryptedPassword ||
        !passwordData.deviceTuyaUuid
      ) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      }
      const addDeviceObj: addDeviceObjectInterface = {
        ...addDoorLockDto,
        ...passwordData,
      };
      const createPass = await this.addOnlineTemporaryPasswordTuya(
        addDeviceObj,
        passwordData.deviceTuyaUuid,
      );

      if (!createPass.success) {
        throw new HttpException(createPass.msg, HttpStatus.BAD_REQUEST);
      }
      return {
        id: createPass.result.id,
        deviceTuyaUuid: passwordData.deviceTuyaUuid,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Error adding  online temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getTicketAndEncryptedPassword(
    doorLockUuid: string,
    passwordPlan: string,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const ticketDetails = await this.createDoorLockTicketTuya(
        deviceDetails.deviceTuyaUuid,
      );

      if (!ticketDetails.result.ticket_id || !ticketDetails.result.ticket_key) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      }
      const decrypted =
        this.passwordEncryptionService.generateEncryptedPassword(
          passwordPlan,
          ticketDetails.result.ticket_key,
        );
      return {
        ticketId: ticketDetails.result.ticket_id,
        ticketKey: ticketDetails.result.ticket_key,
        encryptedPassword: decrypted,
        deviceTuyaUuid: deviceDetails.deviceTuyaUuid,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Error processing the request',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async createDoorLockTicketTuya(
    deviceUuid: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.0/smart-lock/devices/${deviceUuid}/password-ticket`;
      const response = await this.tuya.request({
        method: 'POST',
        path,
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        'Error creating door lock ticket from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async addOnlineTemporaryPasswordTuya(
    addDeviceObj: addDeviceObjectInterface,
    doorLockUuid: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.0/devices/${doorLockUuid}/door-lock/temp-password`;
      let scheduleList;
      if (addDeviceObj.scheduleList.length > 0) {
        scheduleList = addDeviceObj.scheduleList.map((schedule) => ({
          effective_time: this.timeToMinutes(schedule.effectiveTime),
          invalid_time: this.timeToMinutes(schedule.invalidTime),
          working_day: this.getWorkingDayValue(schedule.workingDay),
        }));
      }

      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          name: addDeviceObj.name,
          password: addDeviceObj.encryptedPassword,
          effective_time: addDeviceObj.effectiveTime,
          invalid_time: addDeviceObj.invalidTime,
          password_type: 'ticket',
          ticket_id: addDeviceObj.ticketId,
          ...(addDeviceObj.scheduleList.length > 0 && {
            schedule_list: scheduleList,
          }),

          type: EnableDisableStatusEnum.DISABLED,
        },
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        error.msg || 'Error adding online temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  getWorkingDayValue(days) {
    // Array representing the days of the week
    const weekDays = [
      DaysEnum.SAT,
      DaysEnum.FRI,
      DaysEnum.THU,
      DaysEnum.WED,
      DaysEnum.TUE,
      DaysEnum.MON,
      DaysEnum.SUN,
    ];

    // Initialize a binary string with 7 bits
    let binaryString = '0000000';

    // Iterate through the input array and update the binary string
    days.forEach((day) => {
      const index = weekDays.indexOf(day);
      if (index !== -1) {
        // Set the corresponding bit to EnableDisableStatusEnum.ENABLED
        binaryString =
          binaryString.substring(0, index) +
          EnableDisableStatusEnum.ENABLED +
          binaryString.substring(index + 1);
      }
    });

    // Convert the binary string to an integer
    const workingDayValue = parseInt(binaryString, 2);

    return workingDayValue;
  }
  getDaysFromWorkingDayValue(workingDayValue) {
    // Array representing the days of the week
    const weekDays = [
      DaysEnum.SAT,
      DaysEnum.FRI,
      DaysEnum.THU,
      DaysEnum.WED,
      DaysEnum.TUE,
      DaysEnum.MON,
      DaysEnum.SUN,
    ];

    // Convert the integer to a binary string and pad with leading zeros to ensure 7 bits
    const binaryString = workingDayValue
      .toString(2)
      .padStart(7, EnableDisableStatusEnum.DISABLED);

    // Initialize an array to hold the days of the week
    const days = [];

    // Iterate through the binary string and weekDays array
    for (let i = 0; i < binaryString.length; i++) {
      if (binaryString[i] === EnableDisableStatusEnum.ENABLED) {
        days.push(weekDays[i]);
      }
    }

    return days;
  }
  timeToMinutes(timeStr) {
    try {
      // Special case for "24:00"
      if (timeStr === CommonHours.TWENTY_FOUR) {
        return CommonHourMinutes.TWENTY_FOUR;
      }

      // Regular expression to validate the 24-hour time format (HH:MM)
      const timePattern = /^([01]\d|2[0-3]):([0-5]\d)$/;
      const match = timeStr.match(timePattern);

      if (!match) {
        throw new Error('Invalid time format');
      }

      // Extract hours and minutes from the matched groups
      const hours = parseInt(match[1], 10);
      const minutes = parseInt(match[2], 10);

      // Calculate the total minutes
      const totalMinutes = hours * 60 + minutes;

      return totalMinutes;
    } catch (error) {
      throw new HttpException(error.message, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  minutesToTime(totalMinutes) {
    try {
      if (
        typeof totalMinutes !== 'number' ||
        totalMinutes < 0 ||
        totalMinutes > CommonHourMinutes.TWENTY_FOUR
      ) {
        throw new Error('Invalid minutes value');
      }

      if (totalMinutes === CommonHourMinutes.TWENTY_FOUR) {
        return CommonHours.TWENTY_FOUR;
      }

      const hours = Math.floor(totalMinutes / 60);
      const minutes = totalMinutes % 60;

      const formattedHours = String(hours).padStart(
        2,
        EnableDisableStatusEnum.DISABLED,
      );
      const formattedMinutes = String(minutes).padStart(
        2,
        EnableDisableStatusEnum.DISABLED,
      );

      return `${formattedHours}:${formattedMinutes}`;
    } catch (error) {
      throw new HttpException(error.message, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  async getDeviceByDeviceUuid(
    deviceUuid: string,
    withProductDevice: boolean = true,
  ) {
    try {
      return await this.deviceRepository.findOne({
        where: {
          uuid: deviceUuid,
          isActive: true,
        },
        ...(withProductDevice && { relations: ['productDevice'] }),
      });
    } catch (error) {
      throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
    }
  }

  async updateOfflineTemporaryPassword(
    updateDoorLockOfflineTempDto: UpdateDoorLockOfflineTempDto,
    doorLockUuid: string,
    passwordId: string,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const updateOfflinePass = await this.updateOfflineTemporaryPasswordTuya(
        deviceDetails.deviceTuyaUuid,
        updateDoorLockOfflineTempDto,
        passwordId,
      );
      if (!updateOfflinePass.success) {
        throw new HttpException(updateOfflinePass.msg, HttpStatus.BAD_REQUEST);
      }
      return {
        result: updateOfflinePass.result,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Error updating offline temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async updateOfflineTemporaryPasswordTuya(
    doorLockUuid: string,
    updateDoorLockOfflineTempDto: UpdateDoorLockOfflineTempDto,
    passwordId: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.0/cloud/lock/${doorLockUuid}/door-lock/offline-temp-password/${passwordId}`;

      const response = await this.tuya.request({
        method: 'PUT',
        path,
        body: {
          password_name: updateDoorLockOfflineTempDto.name,
        },
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        'Error updating offline temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async openDoorLock(
    doorLockUuid: string,
  ): Promise<deleteTemporaryPasswordInterface> {
    try {
      // Fetch ticket and encrypted password
      const { ticketKey, encryptedPassword, deviceTuyaUuid, ticketId } =
        await this.getTicketAndEncryptedPassword(
          doorLockUuid,
          defaultDoorLockPass,
        );

      // Validate ticket and device Tuya UUID
      if (!ticketKey || !encryptedPassword || !deviceTuyaUuid) {
        throw new HttpException(
          'Invalid Ticket or Device UUID',
          HttpStatus.NOT_FOUND,
        );
      }

      // Retrieve device details
      const deviceDetails = await this.getDeviceByDeviceUuid(doorLockUuid);

      // Validate device details
      if (!deviceDetails?.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      }

      if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }

      // Construct the path for the Tuya API request
      const path = `/v1.0/devices/${deviceDetails.deviceTuyaUuid}/door-lock/password-free/open-door`;

      // Make the Tuya API request to open the door
      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: { ticket_id: ticketId },
      });

      return response as deleteTemporaryPasswordInterface;
    } catch (error) {
      const status = error.status || HttpStatus.INTERNAL_SERVER_ERROR;
      const message = error.message || 'Error opening door lock from Tuya';

      throw new HttpException(message, status);
    }
  }
}
