import { TuyaService } from '@app/common/integrations/tuya/services/tuya.service';

import {
  HttpException,
  HttpStatus,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { GetSpaceParam } from '../dtos';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';

import { ValidationService } from './space-validation.service';
import { ProductType } from '@app/common/constants/product-type.enum';
import { BatteryStatus } from '@app/common/constants/battery-status.enum';
import { DeviceService } from 'src/device/services';
import { SpaceRepository } from '@app/common/modules/space';
import { DeviceEntity } from '@app/common/modules/device/entities';
import { SpaceEntity } from '@app/common/modules/space/entities/space.entity';
import { GetDevicesBySpaceDto } from '../dtos/device.space.dto';
import { addSpaceUuidToDevices } from '@app/common/util/device-utils';

@Injectable()
export class SpaceDeviceService {
  constructor(
    private readonly tuyaService: TuyaService,
    private readonly validationService: ValidationService,
    private readonly deviceService: DeviceService,
    private readonly spaceRepository: SpaceRepository,
  ) {}

  async listDevicesInSpace(
    params: GetSpaceParam,
    query: GetDevicesBySpaceDto,
  ): Promise<BaseResponseDto> {
    const { spaceUuid, communityUuid, projectUuid } = params;
    const { productType } = query;
    try {
      // Validate community, project, and fetch space including devices in a single query
      const space = await this.validationService.fetchSpaceDevices(spaceUuid);

      if (!space) {
        throw new HttpException('Space not found', HttpStatus.NOT_FOUND);
      }

      const spaceHierarchy =
        await this.validationService.getParentHierarchy(space);

      const orderedHierarchy = [space, ...spaceHierarchy.reverse()];

      // Fetch Tuya details for each device in parallel using Promise.allSettled
      const deviceDetailsPromises = space.devices.map((device) =>
        this.fetchDeviceDetails(device, orderedHierarchy),
      );

      const detailedDevices = (await Promise.allSettled(deviceDetailsPromises))
        .filter((result) => result.status === 'fulfilled' && result.value)
        .map((result) => (result as PromiseFulfilledResult<any>).value);
      console.log('detailedDevices', detailedDevices);
      if (productType) {
        const devicesFilterd = detailedDevices.filter(
          (device) => device.productType === productType,
        );
        if (devicesFilterd.length === 0) {
          return new SuccessResponseDto({
            message: `No ${productType} devices found for ${spaceUuid ? 'space' : 'community'}`,
            data: [],
            statusCode: HttpStatus.CREATED,
          });
        }
        return new SuccessResponseDto({
          data: devicesFilterd,
          message: 'Successfully retrieved list of devices.',
        });
      }
      return new SuccessResponseDto({
        data: detailedDevices,
        message: 'Successfully retrieved list of devices.',
      });
    } catch (error) {
      console.error('Error listing devices in space:', error);
      throw new HttpException(
        error.message || 'Failed to retrieve devices in space',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  private async fetchDeviceDetails(device: any, orderedHierarchy: any[]) {
    try {
      // Fetch Tuya details in parallel
      const [tuyaDetails, tuyaDeviceStatusResponse] = await Promise.all([
        this.deviceService.getDeviceDetailsByDeviceIdTuya(
          device.deviceTuyaUuid,
        ),
        this.tuyaService.getDevicesInstructionStatusTuya(device.deviceTuyaUuid),
      ]);

      const tuyaStatusList =
        tuyaDeviceStatusResponse?.result?.[0]?.status || [];

      return {
        spaces: orderedHierarchy.map((space) => ({
          uuid: space.uuid,
          spaceName: space.spaceName,
        })),
        uuid: device.uuid,
        deviceTuyaUuid: device.deviceTuyaUuid,
        productUuid: device.productDevice?.uuid || null,
        productType: device.productDevice?.prodType || null,
        isActive: device.isActive,
        updatedAt: device.updatedAt,
        deviceTag: device.tag,
        subspace: device.subspace,
        ...tuyaDetails,
        ...(this.extractBatteryStatus(
          device.productDevice?.prodType,
          tuyaStatusList,
        ) !== null && {
          battery: this.extractBatteryStatus(
            device.productDevice?.prodType,
            tuyaStatusList,
          ),
        }),
        status: tuyaStatusList,
      };
    } catch (error) {
      console.warn(
        `Skipping device ${device.deviceTuyaUuid} due to error: ${error.message}`,
      );
      return null;
    }
  }

  private extractBatteryStatus(
    deviceType: string,
    tuyaStatus: any[],
  ): number | null {
    const batteryCodes = {
      [ProductType.DL]: BatteryStatus.RESIDUAL_ELECTRICITY,
      [ProductType.DS]: BatteryStatus.BATTERY_PERCENTAGE,
      [ProductType.WL]: BatteryStatus.BATTERY_PERCENTAGE,
      [ProductType.SOS]: BatteryStatus.BATTERY_PERCENTAGE,
    };

    const batteryCode = batteryCodes[deviceType];
    if (!batteryCode) return null;

    const batteryStatus = tuyaStatus.find(
      (status) => status.code === batteryCode,
    );
    return batteryStatus ? batteryStatus.value : null;
  }
  async getAllDevicesBySpace(spaceUuid: string): Promise<DeviceEntity[]> {
    const space = await this.spaceRepository.findOne({
      where: { uuid: spaceUuid },
      relations: ['children', 'devices', 'devices.productDevice'],
    });

    if (!space) {
      throw new NotFoundException('Space not found');
    }

    const allDevices: DeviceEntity[] = [];
    allDevices.push(...addSpaceUuidToDevices(space.devices, space.uuid));

    // Recursive fetch function
    const fetchChildren = async (parentSpace: SpaceEntity) => {
      const children = await this.spaceRepository.find({
        where: { parent: { uuid: parentSpace.uuid } },
        relations: ['children', 'devices', 'devices.productDevice'],
      });

      for (const child of children) {
        allDevices.push(...addSpaceUuidToDevices(child.devices, child.uuid));

        if (child.children.length > 0) {
          await fetchChildren(child);
        }
      }
    };

    // Start recursive fetch
    await fetchChildren(space);

    return allDevices;
  }
}
