import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { HttpService } from '@nestjs/axios';
import { ConfigService } from '@nestjs/config';
import { firstValueFrom } from 'rxjs';
import { GetWeatherDetailsDto } from '../dto/get.weather.dto';
import { calculateAQI } from '@app/common/util/calculate.aqi';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';

@Injectable()
export class WeatherService {
  private readonly weatherApiUrl: string;
  constructor(
    private readonly configService: ConfigService,
    private readonly httpService: HttpService,
  ) {
    this.weatherApiUrl = this.configService.get<string>('WEATHER_API_URL');
  }

  async fetchWeatherDetails(
    query: GetWeatherDetailsDto,
  ): Promise<BaseResponseDto> {
    try {
      const { lat, lon } = query;
      const weatherApiKey = this.configService.get<string>(
        'OPEN_WEATHER_MAP_API_KEY',
      );
      const url = `${this.weatherApiUrl}/current.json?key=${weatherApiKey}&q=${lat},${lon}&aqi=yes`;

      const response = await firstValueFrom(this.httpService.get(url));
      const pm2_5 = response.data.current.air_quality.pm2_5; // Raw PM2.5 (µg/m³)

      return new SuccessResponseDto({
        message: `Weather details fetched successfully`,
        data: {
          aqi: calculateAQI(pm2_5), // Converted AQI (0-500)
          temperature: response.data.current.temp_c,
          humidity: response.data.current.humidity,
        },
        statusCode: HttpStatus.OK,
      });
    } catch (error) {
      console.log(`Error fetching weather data: ${error}`);

      throw new HttpException(
        `Api can't handle these lat and lon values`,
        error.response?.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
